-- Lua by JugadorXEI
-- Add "Lua.speedpadstoring = true" into your map header to enable speed pad storing.
-- Give Speed Pad linedefs the "No Tails" flag to prevent players from having their view changed.

local LOCALVERSION = 2
if speedpadutils_ver ~= nil and 
	LOCALVERSION <= speedpadutils_ver then return end

freeslot("sfx_spdsa2")

local SPEEDPAD			= 1280
local LINEDEF_SPEEDPAD 	= 4
local DONTSNAPVIEW  	= ML_NOTAILS

local shouldSpeedPadStore = false

local noViewSnapTable = {}
local areThereElementsInViewSnap = false


local function fillSpeedPadTable()
	noViewSnapTable = {}

	for line in lines.iterate do
		if line.valid and line.special == LINEDEF_SPEEDPAD then
			local shouldViewNotSnap = (line.flags & DONTSNAPVIEW) == DONTSNAPVIEW
			if shouldViewNotSnap then
				-- print("added no snap to view speed pad: " + line.tag)
				table.insert(noViewSnapTable, line.tag, true)
				areThereElementsInViewSnap = true
			end
		end
	end
end

local function speedPadUtils(mobj)
	if speedpadutils_ver > LOCALVERSION then return end

	if leveltime == 1 then
		shouldSpeedPadStore =
			mapheaderinfo[gamemap] == nil or
			mapheaderinfo[gamemap].speedpadstoring ~= nil 	and 
			mapheaderinfo[gamemap].speedpadstoring:lower() == "true"

		fillSpeedPadTable()
	end

	-- if we're doing nothing, don't execute any logic
	if areThereElementsInViewSnap == false and shouldSpeedPadStore == false then
		return
	end

	local player 		= mobj.player

	local speedPadSector = P_PlayerTouchingSectorSpecial(player, 3, 5)
	local speedPadFoF 	= P_ThingOnSpecial3DFloor(mobj)

	local isPlayerInSectorSpeedPad =
		speedPadSector ~= nil and
		P_IsObjectOnGround(mobj) == true

	local isFoFSpeedPad = false

	if speedPadFoF ~= nil then
		isFoFSpeedPad = (speedPadFoF.special & SPEEDPAD) == SPEEDPAD 
	end

	if (isPlayerInSectorSpeedPad == true 	or
		isFoFSpeedPad 			== true)	and
		(mobj.dashPadStoring 	== 0 		or
		mobj.dashPadStoring 	== nil) 	then
		
		if shouldSpeedPadStore and 	mobj.lastDriftAngle ~= 0 then
			player.kartstuff[k_drift] 		= mobj.lastDriftAngle
			player.kartstuff[k_driftcharge] = mobj.lastDriftValue
		end
		
		local sectorToCheck =
			isFoFSpeedPad and speedPadFoF or speedPadSector

		if noViewSnapTable[sectorToCheck.tag] then
			mobj.angle = mobj.lastSpeedPadAngle
			-- print("going to where you were lookin' now")
			
			S_StopSoundByID(mobj, sfx_spdpad)
			S_StartSound(mobj, sfx_spdsa2)
		end

		mobj.dashPadStoring = 3

	elseif mobj.dashPadStoring ~= nil and
		mobj.dashPadStoring > 0 then
			
		mobj.dashPadStoring = $ - 1
	else
		mobj.lastDriftAngle = player.kartstuff[k_drift]
		mobj.lastDriftValue = player.kartstuff[k_driftcharge]
		mobj.lastSpeedPadAngle = mobj.angle
	end

end

addHook("MobjThinker", speedPadUtils, MT_PLAYER)
addHook("NetVars", function(host)
	shouldSpeedPadStore 		= host(shouldSpeedPadStore)
	areThereElementsInViewSnap 	= host(areThereElementsInViewSnap)

	if areThereElementsInViewSnap == true then
		noViewSnapTable = host(noViewSnapTable)
	end
end)

rawset(_G, "speedpadutils_ver", LOCALVERSION) -- The script ran, don't run it again